/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation. Portions created by Netscape are
 * Copyright (C) 1998-1999 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributors:
 *  Stephen Bounds <guruj@mbox.com.au>
 *  Chip Clark <chipc@netscape.com>
 *  Seth Spitzer <sspitzer@netscape.com>
 */

/*
 * TODO: remove DEBUG code lines
 */

/* NB:
 * requires "chrome://venkman/content/tree-utils.js" preloaded in XUL
 */

/* ------ Custom View definition ------ */

var prefsTree = new XULTreeView();

function PrefsRecord (pref, lock, type, value, hint) 
{
    this.setColumnPropertyName ("prefCol", "pref");
    this.setColumnPropertyName ("lockCol",  "lock");    
    this.setColumnPropertyName ("typeCol",  "type");    
    this.setColumnPropertyName ("valueCol", "value");    
    this.setColumnPropertyName ("hintCol", "hint");    
    this.pref = pref;
    this.lock = lock;
    this.type = type;
    this.value = value;
    this.hint = hint;
}

PrefsRecord.prototype.getCellProperties = 
    function prf_cellprops(index, col, prop) {};

PrefsRecord.prototype = new XULTreeViewRecord(prefsTree.share);

/* ----- Data generation variables/constants ----- */

// Load string constants from properties bundle
var gBundle = document.getElementById("bundle");

// Preference status & type information lookup
var gLockStrs = [gBundle.getString("statusDefault"),
                 gBundle.getString("statusUser"),
                 gBundle.getString("statusLocked")];

const kDefault = 0;  const kUserSet = 1;  const kLocked = 2;

var gTypeStrs = [gBundle.getString("typeChar"),
                 gBundle.getString("typeInt"),
                 gBundle.getString("typeBool")];

const kStrType = 0;  const kIntType = 1;  const kBoolType = 2;

/* ----- Load RDF files (hints & enums) ----- */

/* NB: enums not done yet */

// The rdf service
const RDF_URI = '@mozilla.org/rdf/rdf-service;1';
var RDF = Components.classes[RDF_URI].getService();
RDF = RDF.QueryInterface(Components.interfaces.nsIRDFService);

// constants identifying the hints RDF
const HINT_URI = "chrome://preferential/locale/preferential-descriptions.rdf";
const HINT_ID = "http://preferential.mozdev.org/HINT-rdf#";
const HINT_VALUE_ID = HINT_ID + 'value';
const HINT_TYPE_ID = HINT_ID + 'type';
const HINT_PREFIX = "urn:preferential:hint:";

var hint_rds = RDF.GetDataSource(HINT_URI);

/* ----- Event Trigger Functions ----- */

function onPrefLoad()
{
    // load up progress meter
    progressBar(-1);
    // load tree next
    setTimeout('prefLoadTree()', 100);
}

function prefLoadTree()
{
    var prefCount = {value:0};
    var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var prefBranch = prefService.getBranch(null);
    var prefArray = prefBranch.getChildList("" , prefCount);
    var prefName, prefType, prefValue, prefIndex, prefLockState;

    var i = 0;   // Counter for iterating through preferences
    var x = 0;   // This is an internal counter for iterating through
                 // headings 

    // these arrays contain preference hierarchy information to determine
    // when new headings are required to be started/finished
    var thisHierarchy = new Array();
    var lastHierarchy = new Array();

    // max number of splits for preference hierarchies
    const splitLimit = 4;

    // contains the tree items currently being worked on in the hierarchy
    var level = new Array(splitLimit);
    
    // Once array is sorted, we can create hierarchy trees based upon
    // similarities and finding '.'s
    prefArray.sort();

    for (i = 0; i < prefCount.value; i++) 
    {
        // DEBUGGING flag
        var _DEBUG = false;

	  // avoid displaying "private" preferences
        if((prefArray[i].indexOf("capability", 0) + 1) > 0) continue;

        // generate tree row values
        if (prefBranch.prefIsLocked(prefArray[i]))
          prefLockState = gLockStrs[kLocked];
        else if(prefBranch.prefHasUserValue(prefArray[i]))
          prefLockState = gLockStrs[kUserSet];
        else
          prefLockState = gLockStrs[kDefault];

        const nsIPrefBranch = Components.interfaces.nsIPrefBranch;

        switch (prefBranch.getPrefType(prefArray[i])) {
            case nsIPrefBranch.PREF_INT:
                prefType = gTypeStrs[kIntType];
                // convert to a string
                prefValue = "" + prefBranch.getIntPref(prefArray[i]);
                    break;
            case nsIPrefBranch.PREF_BOOL:
                prefType = gTypeStrs[kBoolType];
                // convert to a string
                if (prefBranch.getBoolPref(prefArray[i]))
                  prefValue = "true";
                else
                  prefValue = "false";
                    break;
            case nsIPrefBranch.PREF_STRING:
              prefType = gTypeStrs[kStrType];
              prefValue = htmlEscape(prefBranch.getCharPref(prefArray[i]));
                  break;
            default:
              alert(gBundle.getString(msgErrorElement) + prefArray[i]);
        }

        // copy old hierarchy
        delete lastHierarchy;
        for (x=0; x<thisHierarchy.length; x++) {
            lastHierarchy[x] = thisHierarchy[x];
        }

        // generate new hierarchy        
        delete thisHierarchy;
        thisHierarchy = htmlEscape(prefArray[i]).split('.',splitLimit);
        for (x=1; x<thisHierarchy.length; x++) {
            thisHierarchy[x] = thisHierarchy[x-1] + '.' + thisHierarchy[x];
        }

        if (_DEBUG) alert('this:\n' + 
                 + thisHierarchy[0] + '\n' + thisHierarchy[1] + '\n'
                 + thisHierarchy[2] + '\n' + thisHierarchy[3] + '\n'
     + 'last:\n' + lastHierarchy[0] + '\n' + lastHierarchy[1] + '\n\n'
                 + lastHierarchy[2] + '\n' + lastHierarchy[3] + '\n');
        
        // insert new headings as required
        for (x=thisHierarchy.length-1; x >= 0; x--) {
            // unset values for any non-leaf entries
            if (x < thisHierarchy.length-1) {
                prefLockState=""; prefType=""; prefValue="";
            }

            // FURTHER UP THE TREE
            if (thisHierarchy[x] != lastHierarchy[x] || 
                lastHierarchy[x] == undefined) {
                if (_DEBUG) alert('creating record: (overwrite)');
                // generate 'hint' if found in RDF datastore
		    var hint_node = hint_rds.GetTarget(
                     RDF.GetResource(HINT_PREFIX + thisHierarchy[x]),
                     RDF.GetResource(HINT_VALUE_ID),true);
                if (hint_node != undefined)
                    hint = hint_node.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
                else
                    hint = "";
                // create the preferences node
                level[x] = new PrefsRecord(thisHierarchy[x], 
                    prefLockState, prefType, prefValue, hint);
                nextAppendChild = true;
            }

            // append children (if any) to this node
            if (x < (thisHierarchy.length-1)) {
                if (thisHierarchy[x+1] != lastHierarchy[x+1]) {
                    if (_DEBUG) alert(i + ': appending: ' + thisHierarchy[x+1] + ' to ' + thisHierarchy[x]);            
                    if (level[x].childData == undefined)
                        level[x].reserveChildren();
                    level[x].appendChild(level[x+1]);
                }
            }

            // SPECIAL HANDLING FOR ROOT ENTRIES (ie. x=0)
            if (x==0) {
                if (_DEBUG) alert(i + ': handle root');
                // append to root of tree each time base level changes
                if (thisHierarchy[x] != lastHierarchy[x] || 
                   lastHierarchy[x] == undefined) {
                    if (_DEBUG) alert(i + ': append to root: ' + thisHierarchy[x]);
                    prefsTree.childData.appendChild(level[x]);
                }
            }
            if (_DEBUG) alert('done this loop');
        }
    }

    var gTree = document.getElementById("prefsTree");
    gTree.treeBoxObject.view = prefsTree;

    // change progress bar to show as complete
    progressBar(100);
}

function onPrefUnload()
{

}

function PrefNewClick()
{
    var row = prefsTree.childData.locateChildByVisualRow(prefsTree.selectedIndex);
    window.openDialog("chrome://preferential/content/preferential-new.xul",
                      "newPrefDialog","chrome,modal",row.pref,row.lock,row.type,row.value,gBundle);
}

function PrefEditClick()
{
    var row = prefsTree.childData.locateChildByVisualRow(prefsTree.selectedIndex);
    
    // boolean preferences get an enumerated window
    if (row.type == gTypeStrs[kBoolType]) {
        var enumArray = Array(2);
        enumArray[0] = {label:gBundle.getString("constTrue"), value:-1}; 
        enumArray[1] = {label:gBundle.getString("constFalse"), value:0};
        window.openDialog("chrome://preferential/content/preferential-edit-enum.xul",
                      "editPrefDialog","chrome,modal",row.pref,row.lock,row.type,row.value,row.hint,enumArray,gBundle);
    }
    // TODO: if RDF enum list found, open enumerated window
    else {
        window.openDialog("chrome://preferential/content/preferential-edit.xul",
                              "editPrefDialog","chrome,modal",row.pref,row.lock,row.type,row.value,row.hint,gBundle);
    }

}

function PrefDeleteClick()
{
    var row = prefsTree.childData.locateChildByVisualRow(prefsTree.selectedIndex);
    window.openDialog("chrome://preferential/content/preferential-delete.xul",
                      "deletePrefDialog","chrome,modal",row.pref,row.lock,row.type,row.value,row.hint,gBundle);
}

function PrefHelpAboutClick()
{
    window.openDialog("chrome://preferential/content/preferential-about.xul","aboutDialog","chrome");
}

function PrefExpandClick()
{
    var row = prefsTree.childData.locateChildByVisualRow(prefsTree.selectedIndex);
    if (row.isContainerOpen == false) {
        prefsTree.toggleOpenState(prefsTree.selectedIndex);
    }
}

function PrefCollapseClick()
{
    var row = prefsTree.childData.locateChildByVisualRow(prefsTree.selectedIndex);
    if (row.isContainerOpen == true) {
        prefsTree.toggleOpenState(prefsTree.selectedIndex);
    }
}

function PrefExpandAllClick()
{
    // Show undetermined progress bar
    progressBar(-1);

    var rowIndex = prefsTree.selectedIndex;
    var startIndex = rowIndex;
    // get initial indentation level
    var initialIndent = prefsTree.getLevel(rowIndex);

    // go down through each row and expand branches as we go until 
    // we drop below our original indentation level
    do {
        var row = prefsTree.childData.locateChildByVisualRow(rowIndex);
        if (("childData" in row) && row.isContainerOpen == false) {
            prefsTree.toggleOpenState(rowIndex);
        }
        rowIndex++;
    } while (prefsTree.getLevel(rowIndex) > initialIndent)

    // Finished!
    progressBar(100);
}

function PrefCollapseAllClick()
{
    // Show undetermined progress bar
    progressBar(-1);

    var rowIndex = prefsTree.selectedIndex;
    var startIndex = rowIndex;
    var row = prefsTree.childData.locateChildByVisualRow(rowIndex);
    // get initial indentation level
    var initialIndent = prefsTree.getLevel(rowIndex++);
    // find point at which we drop below our original indentation level;
    while (prefsTree.getLevel(rowIndex) > initialIndent)
        row = prefsTree.childData.locateChildByVisualRow(rowIndex++);
    // next, work back upwards, collapsing as we go until we reach starting
    // point again
    row = prefsTree.childData.locateChildByVisualRow(--rowIndex);  
    while (rowIndex >= startIndex && row != undefined) {
        if (("childData" in row) && row.isContainerOpen == true) {
            prefsTree.toggleOpenState(rowIndex);
        }        
        var row = prefsTree.childData.locateChildByVisualRow(--rowIndex);
    }

    // Finished!
    progressBar(100);
}

function PrefViewDetailsToggle()
{
    menuItem = document.getElementById('prefsViewDetails');

    if (menuItem.checked == false) {
        menuItem.checked = true;
        window.open("chrome://preferential/content/preferential-details.xul",
                              "prefDetails","chrome");
    }
    else {
        menuItem.checked = false;
        window.close("prefDetails");
    }
}

/* --------- UTILITY FUNCTIONS --------- */

function htmlEscape(s) 
{ 
    s = s.replace(/\&/g, "&amp;"); 
    s = s.replace(/\>/g, "&gt;"); 
    s = s.replace(/\</g, "&lt;"); 
    return s; 
}

function progressBar(percent)
{
    if (percent < 100)
        document.getElementById('progressLabel').value = gBundle.getString("msgProgressBuilding");
    else 
        document.getElementById('progressLabel').value = gBundle.getString("msgProgressDone");

    if (percent >= 0) {
        document.getElementById('progressTask').mode = 'determined';
        document.getElementById('progressTask').value = percent;
    }
    else {
        document.getElementById('progressTask').mode = 'undetermined';
    }
}